import { useMemo, useState } from 'react';
import './App.css';
import { DayPilot, DayPilotCalendar, DayPilotNavigator } from '@daypilot/daypilot-lite-react';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import "./Calendar.css";

function App() {
  // TODO: Вынести некоторые параметры в настройки, например partnerId.
  // TODO: Сделать прокидку цены бронирования
  // TODO: Сделать четкую форму, а не говно
  const [calendar, setCalendar] = useState<DayPilot.Calendar>();
  const [currentChoosedEvent, setCurrentChoosedEvent] = useState<DayPilot.EventData>();
  const [scheduleData, setScheduleData] = useState<Door[]>([]);
  const [hiddenArr, setHiddenArr] = useState<boolean[]>([]);
  const [email, setEmail] = useState<string>("")
  const [router, setRouter] = useState<Router>({payment: false, rent: true});
  const [rentId, setRentId] = useState<string>();
  const [startDate, setStartDate] = useState(new DayPilot.Date(new Date(getMondayOfCurrentWeek())));
  useMemo(() => {
    fetch("http://localhost:8080/public/slots/4bfc6f62-588f-4d92-bf45-96e53e0cb291")
    .then(res => res.json())
    .then(res => {
      let castedResult = res as DoorsResponse;
      setScheduleData(castedResult.doors);
      setHiddenArr(castedResult.doors.map(d => true))
    });
  }, [])

  return (
    <div className="App">
      <div className='App-Rent' hidden={!router.rent}>
        <div hidden={hiddenArr.every(h => h)} className='App-Form'>
          <div className='App-Calendar'>
            <DayPilotNavigator 
              onTimeRangeSelect={args => {
                setStartDate(args.day)
              }}
              selectionDay={startDate}
            />
            <DayPilotCalendar
              viewType="Week"
              controlRef={setCalendar}
              durationBarVisible={false}
              startDate={startDate}
              onTimeRangeSelected={args => {
                const e = {
                  start: args.start,
                  end: args.end,
                  id: DayPilot.guid(),
                  text: "Выбрано",
                  backColor: "#2e78d6"
                };
                currentChoosedEvent && calendar?.events.remove(currentChoosedEvent?.id);
                setCurrentChoosedEvent(e);
                calendar?.events.add(e);
              }} />
          </div>
          <div className='App-Fields'>
            <div className='App-Form-Element'>
              <TextField value={email} onChange={e => {setEmail(e.target.value)}} label='Email' />
            </div>
          </div>
        </div>
        <div className={"App-Doors".concat(hiddenArr.some(h => !h) ? " App-Cancel-Buttons" : "")}>
          {scheduleData.map((door, ix) => {
            return (
              <div key={ix} className='App-Buttons'>
                <div hidden={hiddenArr[ix]} className='App-Manage-Button'>
                  <Button variant='contained' onClick={() => {
                    fetch("http://localhost:8080/public/book", {
                      method: 'POST',
                      body: JSON.stringify({
                        startDateTime: currentChoosedEvent?.start.toString(),
                        endDateTime: currentChoosedEvent?.end.toString(),
                        doorId: door.id,
                        clientEmail: email
                      }),
                      headers: [
                        ["Content-Type", "application/json"],
                      ]
                    }).then(res => res.json())
                    .then(res => {
                      const castedResponse = res as RentResponse;
                      setRentId(castedResponse.rentId);
                      setRouter({rent: false, payment: true});
                    })
                  }}>Забронировать</Button>
                </div>
                <div hidden={hiddenArr[ix]} className='App-Manage-Button'>
                  <Button variant='outlined' onClick={() => {
                    let newArr = hiddenArr.slice();
                    newArr[ix] = newArr[ix] != true;
                    setHiddenArr(newArr);
                    setEmail("");
                  }}>Отмена</Button>
                </div>
                <div hidden={hiddenArr.some(h => !h)} className='App-Manage-Button'>
                  <Button variant='contained' onClick={() => {
                    let newArr = hiddenArr.slice();
                    newArr[ix] = newArr[ix] != true;
                    const itemsToDelete = calendar?.events.list?.map(ev => ev.id);
                    itemsToDelete?.forEach(d => calendar?.events.remove(d));
                    door.slots
                      .filter(s => s.status != "FREE")
                      .forEach(s => {
                        const startDateTime = new DayPilot.Date(new Date(s.date + 'T' + s.startTime), true);
                        const endDateTime = new DayPilot.Date(new Date(s.date + 'T' + s.endTime), true);
                        const e = {
                          start: startDateTime,
                          end: endDateTime,
                          id: DayPilot.guid(),
                          text: mapSlotStatusToScheduleText(s.status),
                          backColor: mapSlotStatusToScheduleBarColor(s.status)
                        };
                        calendar?.events.add(e);
                      });
                    setHiddenArr(newArr)
                  }}>
                    {door.description}
                  </Button>
                </div>
              </div>
            )
          })}
        </div>
      </div>
      <div className='App-Payment' hidden={!router.payment}>
        <Button variant='contained' onClick={() => {
          fetch("http://localhost:8080/public/book/callback", {
            method: "POST",
            body: JSON.stringify({
              rentId: rentId
            }),
            headers: [
              ["Content-Type", "application/json"],
            ]
          })
          .then(() => {
            window.location.reload();
          })
        }}>
          Оплатить
        </Button>
      </div>
    </div>
  );
}

function mapSlotStatusToScheduleText(status: string): string {
  switch (status) {
    case "OUT_OF_WORKING_TIME":
      return "Вне рабочего времени";
    case "BOOKED":
      return "Занято";
    case "FREE":
      return "Свободно";
    default:
      return "";
  }
}

function mapSlotStatusToScheduleBarColor(status: string): string {
  switch (status) {
    case "OUT_OF_WORKING_TIME":
      return "grey";
    case "BOOKED":
      return "#8b0000";
    case "FREE":
      return "";
    default:
      return "";
  }
}

interface DoorsResponse {
  doors: Door[]
}

interface Door {
  id: string
  description: string
  slots: Slot[]
}

interface Slot {
  startTime: string
  endTime: string
  date: string,
  status: string
}

interface Router {
  payment: boolean;
  rent: boolean;
}

interface RentResponse {
  rentId: string;
}

function getMondayOfCurrentWeek() {
  const today = new Date();
  const dayOfWeek = today.getDay();
  
  const daysSinceMonday = (dayOfWeek + 7) % 7;
  
  const monday = new Date(today);
  monday.setDate(today.getDate() - daysSinceMonday);
  
  return monday;
}

export default App;
